#include <SPI.h>
#include "font8x16.h"
#include "watherIcon.h"
#include "ssd1331.h"

SPISettings settings = SPISettings(8000000, MSBFIRST, SPI_MODE3);

// Declaration for SSD1306 display connected using software SPI (default case):
#define ENABLE_CS FALSE
#define OLED_CS    10   // Slave Select(Chip select)
#define OLED_DC     25  // SSD1306 Data/Command
#define OLED_RESET  19  // SSD1306 Reset

// --- SSD1331 COMMADN ---
static uint8_t oledInitCommands[] = { // SSD1331の初期化コマンド群
  0xAE,         // DISPLAYOFF
  0xA0 , 0x32,  // SETREMAP 0x72 / 0x76 : RGB Color / 0x32 -256Colors
  0xA1 , 0x00 , // STARTLINE
  0xA2 , 0x00 , // DISPLAYOFFSET
  0xA4 ,        // NORMALDISPLAY
  0xB0 , 0x0B , // POWERMODE
  0x87 , 0x01 , // MASTERCURRENT : 0x06
  0x81 , 0x91 , // CONTRAST RED  : 0x91
  0x82 , 0x50 , // CONTRAST GREEN: 0x50
  0x83 , 0x7D , // CONTRAST BLUR : 0x7D
  0xAF ,        // DISPLAYON
};

// 初期化
void SSD1331::init() {
  SPI.setDataMode(SPI_MODE0);//SPIモードを「0」に設定・CPOL(クロック位相)=0,CPHA(クロック極性)=0
  SPI.setBitOrder(MSBFIRST);//SPI送受信用のビットオーダーを「MSBFIRST」に設定

  // --- INIT PARAMETA
  fontColorB = 0x00 ; // TEXT BACK GROND COLOR
  fontColorF = 0xFF ; // TEXT FOR GROND COLOR
  fontColorT = 0x1C ; // TEXT FOR GROND COLOR

  // --- SOFTWARE SPI ピン設定
#if ENABLE_CS == TRUE
  pinMode(OLED_CS, OUTPUT);   // Sleva Select
#endif
  pinMode(OLED_DC, OUTPUT);   // SSD1306 Set data/command pin as output
  pinMode(OLED_RESET, OUTPUT); 

  // --- Reset SSD1306
#if ENABLE_CS == TRUE
  digitalWrite(OLED_CS, LOW);
#endif
  digitalWrite(OLED_RESET, HIGH);
  delay(1);                       // VDD goes high at start, pause for 1 ms
  digitalWrite(OLED_RESET, LOW);  // Bring reset low
  delay(10);                      // Wait 10 ms
  digitalWrite(OLED_RESET, HIGH); // Bring out of reset
  delay(100);                     // Wait 10 ms

  // --- SSD1306 初期化
  // OLED設定コマンド送信
  oledSendCommands(sizeof(oledInitCommands), oledInitCommands) ;  // 初期化
  oledSendCommand(0xA4) ; // 描画ON
  oledSendCommand(0xAF) ; // 画面表示
  delay(100);                     // Wait 10 ms

  cls() ;
}

// 画面消去
void SSD1331::cls() {
  oledSendCommand2(0x26, 0x01) ; // 塗りつぶしON
  drowRect(0,0,OLED_WIDTH-1,OLED_HEIGHT-1,0x00000000,0x00000000) ;
}

void SSD1331::drowRect(uint8_t xs,uint8_t ys,uint8_t xe,uint8_t ye,uint32_t lineColor, uint32_t fillColor) {
  // ----- FILL -----
  cmdBuff[0] = 0x22 ; // Rect 
  cmdBuff[1] = xs ;
  cmdBuff[2] = ys ;
  cmdBuff[3] = xe ;
  cmdBuff[4] = ye ;
  
  cmdBuff[5] = (lineColor >> 16) & 0x000000ff ;
  cmdBuff[6] = (lineColor >> 8) & 0x000000ff ;
  cmdBuff[7] = lineColor & 0x000000ff ;

  cmdBuff[8] = (fillColor >> 16) & 0x000000ff ;
  cmdBuff[9] = (fillColor >> 8) & 0x000000ff ;
  cmdBuff[10]= fillColor & 0x000000ff ;

  oledSendCommands(11 , cmdBuff) ;  // DrowLine Command
}

void SSD1331::dispStr(char str[] , uint8_t x , uint8_t y) {
  for (int i=0;str[i] != 0 ; i++) {
      dispFont(str[i],x,y) ;
      x += 8 ;
      if (x+8 >= OLED_WIDTH) {
        x = 0 ;
        y += 16 ;
        if (y + 16 >= OLED_HEIGHT) {
          y = 0 ; 
        }
      }
  }
}

void SSD1331::dispFont(uint8_t code , uint8_t x, uint8_t y) {
  int pos = (code - 0x20) * 16 ;
  memset(SPIBuf,0,8*16) ;
  for (int i=0;i<16;i++) {
    uint8_t data = ssd1306xled_font8x16[pos + i];
    uint8_t bitMast = 0x01 ; 
    for (int j=0;j<8;j++) {
      int fpos = 0 ;
      if (i<8) {
        fpos = j*8+i ;
      } else {
        fpos = (j+8)*8+(i-8) ;
      }
      if ((data & bitMast) == 0) {
        SPIBuf[fpos] = fontColorB ;
      } else {
        SPIBuf[fpos] = fontColorF ;
      }
      bitMast <<= 1 ;
    }
  }

  uint8_t endX = x + 7 ;
  uint8_t endY = y + 15;

  oledSendCommand3(0x15, x, endX) ;
  oledSendCommand3(0x75, y, endY) ;
  delayMicroseconds(500);
#if ENABLE_CS == TRUE
  digitalWrite(OLED_CS, LOW); // SSD1331セレクト
#endif
  digitalWrite(OLED_DC, HIGH); // Data mode
  SPI.transfer(SPIBuf, 8*16);
  digitalWrite(OLED_DC, LOW); // Command mode
#if ENABLE_CS == TRUE
  digitalWrite(OLED_CS, HIGH);  // SSD1331解放
#endif
  delayMicroseconds(500);
}

void SSD1331::dispWatherIcon(int waterId , uint8_t x , uint8_t y) {
  int offsetX = 0 ;
  int offsetY = 0 ;
  
  memset(SPIBuf,0,48*32) ;
  int watherPos ;
  for (watherPos=0; watherCode[watherPos].watherCode != 0; watherPos++) {
    if (watherCode[watherPos].watherCode == waterId) {
      break ;
    }
  }

  int iconPos ;
  for (iconPos = 0 ;watherIcon[iconPos].iconNo != 0 ; iconPos ++) {
    if (watherIcon[iconPos].iconNo == watherCode[watherPos].iconDay) {
      break ;
    }
  }

  int sizeX,sizeY ;
  for (int i=0;i<3;i++) {
    uint8_t *icon = getIconAddr(watherIcon[iconPos].icon[i],sizeX,sizeY) ;
    offsetX = 0 ;
    offsetY = 0 ;
    if (i == 0) {
      if (sizeX == 30 && watherIcon[iconPos].icon[1] == 0) {
        offsetX = 8 ;
        offsetY = 0 ;
      } else
      if (sizeX == 22) {
        offsetX = 1 ;
        offsetY = 5 ;
      }
    } else
    if (i == 1) {
      if (sizeX == 22) {
        offsetX = 22 ;
        offsetY = 5 ;
      } else
      if (sizeX == 16) {
        offsetX = 30 ;
        offsetY = 10 ;
      }
    } else
    if (i == 2) {
      offsetX = 16 ;
      offsetY = 20 ;
    }

    if (icon != NULL) {
      for (int py=0;py<sizeY;py++) {
        for (int px=0;px<sizeX;px++) {
          if (*icon != 0) { 
            SPIBuf[ (offsetY + py) *48 + offsetX + px ] = *icon ;
          }
          icon ++ ;
        }
      }
    }
  }
  // -- 枠 --
  for (int i=0;i<48;i++) {
    SPIBuf[i] = 0x03 ;
    SPIBuf[48*31+i] = 0x03 ;
  }
  for (int i=0;i<32;i++) {
    SPIBuf[48*i] = 0x03 ;
    SPIBuf[48*i+47] = 0x03 ;
  }

  uint8_t endX = x + 47 ;
  uint8_t endY = y + 31;
  oledSendCommand3(0x15, x, endX) ;
  oledSendCommand3(0x75, y, endY) ;
  delayMicroseconds(500);
#if ENABLE_CS == TRUE
  digitalWrite(OLED_CS, LOW); // SSD1331セレクト
#endif
  digitalWrite(OLED_DC, HIGH); // Data mode
  SPI.transfer(SPIBuf, 48*32);
  digitalWrite(OLED_DC, LOW); // Command mode
#if ENABLE_CS == TRUE
  digitalWrite(OLED_CS, HIGH);  // SSD1331解放
#endif
  delayMicroseconds(500);
}

uint8_t* SSD1331::getIconAddr(int iconNo,int &sx,int &sy) {
  uint8_t *icon = NULL ;
  switch(iconNo) {
    case  1: icon = image_data_icon00 ; sx = 16 ; sy =  8 ; break ; // ⇒
    case 10: icon = image_data_icon10 ; sx = 30 ; sy = 30 ; break ; // 晴れ(大)
    case 11: icon = image_data_icon11 ; sx = 22 ; sy = 22 ; break ; // 晴れ(中)
    case 12: icon = image_data_icon12 ; sx = 16 ; sy = 16 ; break ; // 晴れ(小)
    case 20: icon = image_data_icon20 ; sx = 30 ; sy = 30 ; break ; // 曇り(大)
    case 21: icon = image_data_icon21 ; sx = 22 ; sy = 22 ; break ; // 曇り(中)
    case 22: icon = image_data_icon22 ; sx = 16 ; sy = 16 ; break ; // 曇り(小)
    case 30: icon = image_data_icon30 ; sx = 30 ; sy = 30 ; break ; // 雨(大)
    case 31: icon = image_data_icon31 ; sx = 22 ; sy = 22 ; break ; // 雨(中)
    case 32: icon = image_data_icon32 ; sx = 16 ; sy = 16 ; break ; // 雨(小)
    case 40: icon = image_data_icon40 ; sx = 30 ; sy = 30 ; break ; // 雪(大)
    case 41: icon = image_data_icon41 ; sx = 22 ; sy = 22 ; break ; // 雪(中)
    case 42: icon = image_data_icon42 ; sx = 16 ; sy = 16 ; break ; // 雪(小)
    case 50: icon = image_data_icon50 ; sx = 30 ; sy = 30 ; break ; // 暴雨(大)
    case 60: icon = image_data_icon60 ; sx = 30 ; sy = 30 ; break ; // 暴雪(大)
  }
  return icon ;
}

// ------------------------------------------------------
// OLEDに1バイトコマンドを送信
void SSD1331::oledSendCommand(uint8_t command) {
#if ENABLE_CS == TRUE
  digitalWrite(OLED_CS, LOW); // SSD1331セレクト
#endif
  digitalWrite(OLED_DC, LOW); // Command mode
  SPI.transfer(command);
#if ENABLE_CS == TRUE
  digitalWrite(OLED_CS, HIGH);  // SSD1331解放
#endif
}

// OLEDに2バイトコマンドを送信
void SSD1331::oledSendCommand2(uint8_t command1, uint8_t command2) {
#if ENABLE_CS == TRUE
  digitalWrite(OLED_CS, LOW); // SSD1331セレクト
#endif
  digitalWrite(OLED_DC, LOW); // Command mode
  SPI.transfer(command1);
  SPI.transfer(command2);
#if ENABLE_CS == TRUE
  digitalWrite(OLED_CS, HIGH);  // SSD1331解放
#endif
}

// OLEDに3バイトコマンドを送信
void SSD1331::oledSendCommand3(uint8_t command1, uint8_t command2, uint8_t command3) {
#if ENABLE_CS == TRUE
  digitalWrite(OLED_CS, LOW); // SSD1331セレクト
#endif
  digitalWrite(OLED_DC, LOW); // Command mode
  SPI.transfer(command1);
  SPI.transfer(command2);
  SPI.transfer(command3);
#if ENABLE_CS == TRUE
  digitalWrite(OLED_CS, HIGH);  // SSD1331解放
#endif
}

// OLEDに複数のコマンドを送信
void SSD1331::oledSendCommands(int commandLen, uint8_t commands[]) {
#if ENABLE_CS == TRUE
  digitalWrite(OLED_CS, LOW); // SSD1306セレクト
#endif
  digitalWrite(OLED_DC, LOW); // Command mode
  SPI.transfer(commands, commandLen);
#if ENABLE_CS == TRUE
  digitalWrite(OLED_CS, HIGH);  // SSD1306解放
#endif
}
